@@ -15,7 +15,7 @@
 ;;; along with this program; if not, write to the Free Software
 ;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(provide 'vm-page)
+;;(provide 'vm-page)
 
 (defun vm-scroll-forward (&optional arg)
   "Scroll forward a screenful of text.
@@ -506,36 +506,41 @@ Negative arg means scroll forward."
 
 (defun vm-display-xface-fsfemacs ()
   (catch 'done
-    (let ((case-fold-search t) i g h)
-      (if (next-single-property-change (point-min) 'vm-xface)
-	  nil
-	(goto-char (point-min))
-	(if (re-search-forward "^X-Face:" nil t)
-	    (progn
-	      (goto-char (match-beginning 0))
-	      (vm-match-header)
-	      (setq h (vm-matched-header-contents))
-	      (setq g (intern h vm-xface-cache))
-	      (if (boundp g)
-		  (setq g (symbol-value g))
-		(setq i (vm-convert-xface-to-fsfemacs-image-instantiator h))
-		(cond (i
-		       (set g i)
-		       (setq g (symbol-value g)))
-		      (t (throw 'done nil))))
-	      (let ((pos (vm-vheaders-of (car vm-message-pointer)))
-		    o )
-		;; An image must replace the normal display of at
-		;; least one character.  Since we want to put the
-		;; image at the beginning of the visible headers
-		;; section, it will obscure the first character of
-		;; that section.  To display that character we add
-		;; an after-string that contains the character.
-		;; Kludge city, but it works.
-		(setq o (make-overlay (+ 0 pos) (+ 1 pos)))
-		(overlay-put o 'after-string
-			     (char-to-string (char-after pos)))
-		(overlay-put o 'display g))))))))
+    (let ((case-fold-search t) i g h ooo)
+      (setq ooo (overlays-in (point-min) (point-max)))
+      (while ooo
+	(if (overlay-get (car ooo) 'vm-xface)
+	    (delete-overlay (car ooo)))
+	(setq ooo (cdr ooo)))
+      (goto-char (point-min))
+      (if (re-search-forward "^X-Face:" nil t)
+	  (progn
+	    (goto-char (match-beginning 0))
+	    (vm-match-header)
+	    (setq h (vm-matched-header-contents))
+	    (setq g (intern h vm-xface-cache))
+	    (if (boundp g)
+		(setq g (symbol-value g))
+	      (setq i (vm-convert-xface-to-fsfemacs-image-instantiator h))
+	      (cond (i
+		     (set g i)
+		     (setq g (symbol-value g)))
+		    (t (throw 'done nil))))
+	    (let ((pos (vm-vheaders-of (car vm-message-pointer)))
+		  o )
+	      ;; An image must replace the normal display of at
+	      ;; least one character.  Since we want to put the
+	      ;; image at the beginning of the visible headers
+	      ;; section, it will obscure the first character of
+	      ;; that section.  To display that character we add
+	      ;; an after-string that contains the character.
+	      ;; Kludge city, but it works.
+	      (setq o (make-overlay (+ 0 pos) (+ 1 pos)))
+	      (overlay-put o 'vm-xface t)
+	      (overlay-put o 'evaporate t)
+	      (overlay-put o 'after-string
+			   (char-to-string (char-after pos)))
+	      (overlay-put o 'display g)))))))
 
 (defun vm-convert-xface-to-fsfemacs-image-instantiator (data)
   (let ((work-buffer nil)
@@ -699,7 +704,7 @@ Use mouse button 3 to choose a Web browser for the URL."
      (vm-narrow-for-preview just-passing-through)
      (if (or vm-mime-display-function
 	     (natnump vm-fill-paragraphs-containing-long-lines)
-	     (and vm-display-using-mime
+	     (and vm-display-using-mime vm-auto-decode-mime-messages
 		  (not (vm-mime-plain-message-p (car vm-message-pointer)))))
 	 (let ((layout (vm-mm-layout (car vm-message-pointer))))
 	   (vm-make-presentation-copy (car vm-message-pointer))
@@ -1049,3 +1054,5 @@ exposed and marked as read."
 	(goto-char (vm-extent-start-position e))
       (goto-char old-point)
       (error "No more buttons"))))
+
+(provide 'vm-page)
